#include "mbed.h"
#include "AD9851.h"

extern int gDebug;                  // defined in main.cpp, used for printing verbose-ness level (0-3)

// power up default values
float BaseFreq = 10000000.000;      // base frequency - if frequency
float IfFreq = 0.0;                 // IF frequency
float RealFreq = BaseFreq + IfFreq; // sum of the two frequencies
float RefOsc = 30000000.0;          // reference oscillator frequency
char MultSix = 'A';                 // x1 or x6 frequency oscillator - 0 1 or A only
bool OutpEnab = false;              // output enable flag
//char SerData[5];                    // serial buffer data output

float HzPerCount = 0.006984919;     // calculated frequency in Hz per LSB 0.006984919 or 0.041909516
float AddPerCount = 143.1655765;    // calculated add per LSB 143.1655765 or 23.86092942
float SerCount = 1431655765.0;      // calculated final value of data to AD9851
unsigned int FreqDiv = 0x55555555;  // calculated value of SerCount to be divided into SerData[0-4]
char FortyBitWd = 0x00;             // calculated 5th byte fo 40 bit word to be divided into SerData[0-4]

const char PwrDnFlag = 0x04;        // + power down flag
const char Mult6Flag = 0x01;        // + 180MHz - 30MHz multiplier x6 flag

// DigitalOut _clk(CLK, "_clk");         // serial clk pin
// DigitalOut _sdo(SDO, "_sdo");         // serial data out pin
// DigitalOut _len(LEN, "_len");         // data latch strobe pin

// constructor takes paramters and initializes I2C bus
AD9851::AD9851(PinName CLK, PinName SDO, PinName LEN)
      : _clk(CLK)
      , _sdo(SDO)
      , _len(LEN) 
    {
    _clk = 0;
    _sdo = 0;
    _len = 0;
    OutNewValue();                  // initialize the AD9851
}

// destructor
AD9851::~AD9851() {
}

// Private routines:
// Set new value to AD9851
void AD9851::OutNewValue() {
    unsigned int ShiftOut = FreqDiv;
    unsigned int TestOut = FreqDiv;
    char i = 0;
    for(i=0; i<32; i++) {
        TestOut = ShiftOut & 0xfffffffe;
        if(TestOut==1) {
            _sdo = 1;
        } else {
            _sdo = 0;
        }
        _clk = 1;
        ShiftOut = ShiftOut >> 1;
        _clk = 0;
    }
    ShiftOut = FortyBitWd;
    for(i=0; i<8; i++) {
        TestOut = ShiftOut & 0xfffffffe;
        if(TestOut==1) {
            _sdo = 1;
        } else {
            _sdo = 0;
        }
        _clk = 1;
        ShiftOut = ShiftOut >> 1;
        _clk = 0;
    }
    _len = 1;
    _len = 0;
}